@@ -2,7 +2,7 @@
 ;;
 ;; Author: Vitalie Spinu
 ;; Maintainer: Vitalie Spinu
-;; Copyright (C) 2013-2019, Vitalie Spinu
+;; Copyright (C) 2013-2022  Free Software Foundation, Inc.
 ;; URL: https://github.com/polymode/polymode
 ;; Keywords: emacs
 ;;
@@ -71,8 +71,6 @@ Elements of ALIST that are not conses are ignored."
 
 ;;; Various Wrappers for Around Advice
 
-(defvar *span* nil)
-
 ;; advice doesn't provide named symbols. So we need to define specialized
 ;; wrappers for some key functions (unfinished)
 (defmacro pm-define-wrapp-protected (fun)
@@ -110,8 +108,9 @@ Return new name (symbol). FUN is an unquoted name of a function."
 
 (defun pm-override-output-position (orig-fun &rest args)
   "Restrict returned value of ORIG-FUN to fall into the current span.
-*span* in `pm-map-over-spans` has precedence over span at point.
- ARGS are passed to ORIG-FUN."
+When this function is called from within `pm-map-over-spans' the
+dynamic variable *span* has precedence over the span at point.
+ARGS are passed to ORIG-FUN."
   (if (and polymode-mode pm/polymode)
       (let ((range (or (pm-span-to-range *span*)
                        (pm-innermost-range)))
@@ -123,9 +122,10 @@ Return new name (symbol). FUN is an unquoted name of a function."
 
 (defun pm-override-output-cons (orig-fun &rest args)
   "Restrict returned (beg . end) of ORIG-FUN to fall into the current span.
-*span* in `pm-map-over-spans` has precedence over span at point.
-This will break badly if (point) is not inside expected range.
-ARGS are passed to ORIG-FUN."
+When this function is called from within `pm-map-over-spans' the
+dynamic variable *span* has precedence over span at point. This
+will break badly if (point) is not inside expected range. ARGS
+are passed to ORIG-FUN."
   (if (and polymode-mode pm/polymode)
       (let ((range (or (pm-span-to-range *span*)
                        (pm-innermost-range)))
@@ -142,9 +142,10 @@ ARGS are passed to ORIG-FUN."
 
 (defun pm-narrowed-override-output-cons (orig-fun &rest args)
   "Restrict returned (beg . end) of ORIG-FUN to fall into the current span.
-Run ORIG-FUN with buffer narrowed to span. *span* in
-`pm-map-over-spans` has precedence over span at point. ARGS are
-passed to ORIG-FUN."
+Run ORIG-FUN with buffer narrowed to span. When this function is
+called from within `pm-map-over-spans' the dynamic variable
+*span* has precedence over span at point. ARGS are passed to
+ORIG-FUN."
   (if (and polymode-mode pm/polymode)
       (let ((*span* (or *span* (pm-innermost-span))))
         (pm-with-narrowed-to-span *span*
@@ -153,7 +154,8 @@ passed to ORIG-FUN."
 
 (defun pm-substitute-beg-end (orig-fun beg end &rest args)
   "Execute ORIG-FUN with first BEG and END arguments limited to current span.
-*span* in `pm-map-over-spans` has precedence over span at point.
+When this function is called from within `pm-map-over-spans' the
+dynamic variable *span* has precedence over span at point.
  ARGS are passed to ORIG-FUN."
   (if (and polymode-mode pm/polymode)
       (let* ((pos (if (and (<= (point) end) (>=  (point) beg))
@@ -168,19 +170,163 @@ passed to ORIG-FUN."
 
 (defun pm-execute-narrowed-to-span (orig-fun &rest args)
   "Execute ORIG-FUN narrowed to the current span.
-*span* in `pm-map-over-spans` has precedence over span at point.
- ARGS are passed to ORIG-FUN."
+When this function is called from within `pm-map-over-spans' the
+dynamic variable *span* has precedence over span at point. ARGS
+are passed to ORIG-FUN."
   (if (and polymode-mode pm/polymode)
       (pm-with-narrowed-to-span *span*
         (pm-apply-protected orig-fun args))
     (apply orig-fun args)))
 
+
+
+;;; LSP (lsp-mode and eglot)
+;;
+;; Emacs modifications `after-change-functions' to LSP insertions
+;; https://microsoft.github.io/language-server-protocol/specifications/specification-current/#textDocument_didChange
+;;
+;; INSERT: (50 56 0) means insert 6 chars starting at pos 50
+;;   {"range": {"start": {"line": 1, "character": 0},
+;;              "end"  : {"line": 1, "character": 0}},
+;;              "text": "insert"}
+;;
+;; DELETE: (50 50 6) means delete 6 chars starting at pos 50
+;;   {"range": {"start": {"line": 1, "character": 0},
+;;              "end"  : {"line": 1, "character": 6}},
+;;              "text": ""}
+;;
+;; REPLACE: (50 60 6) means delete 6 chars starting at pos 50, and replace
+;; them with 10 chars
+;;   {"range": {"start": {"line": 1, "character": 0},
+;;              "end" :  {"line": 1, "character": 6}},
+;;              "text": "new-insert"}
+;;
+;; INSERT:
+;;   before-change:(obeg,oend)=(50,50)
+;;   after-change:(nbeg,nend,olen)=(50,56,0)
+;;
+;; DELETE:
+;;   before-change:(obeg,oend)=(50,56)
+;;   after-change:(nbeg,nend,len)=(50,50,6)
+;;
+;; REPLACE:
+;;   before-change:(obeg,oend)=(50,56)
+;;   lsp-on-change:(nbeg,nend,olen)=(50,60,6)
+(defun pm--lsp-buffer-content-document-content-change-event (beg end len)
+  "Make a TextDocumentContentChangeEvent body for BEG to END, of length LEN."
+  (if (zerop len)
+      ;; insertion
+      (pm--lsp-change-event beg end (buffer-substring-no-properties beg end))
+    (if (pm--lsp-simple-change-p beg len)
+        (let ((end-pos pm--lsp-before-change-end-position)
+              (text (buffer-substring-no-properties beg end)))
+          ;; if beg == end deletion, otherwise replacement
+          (pm--lsp-change-event beg end-pos text))
+      (pm--lsp-full-change-event))))
+
+(defun pm--lsp-position (pos)
+  (save-restriction
+    (widen)
+    (save-excursion
+      (goto-char pos)
+      (let ((char (if (eq pm/chunkmode (nth 3 (pm-innermost-span pos)))
+                      (- (point) (line-beginning-position))
+                    0)))
+        (list :line (1- (line-number-at-pos pos))
+              :character char)))))
+
+(defun pm--lsp-change-event (beg end text)
+  (list
+   :range (list
+           :start (if (listp beg) beg (pm--lsp-position beg))
+           :end   (if (listp end) end (pm--lsp-position end)))
+   :text text))
+
+(defun pm--lsp-full-change-event ()
+  (list :text (pm--lsp-buffer-content)))
+
+(defun pm--lsp-buffer-content (&optional beg end)
+  "Get text between BEG and END cleaned from non-current mode content.
+The text from non-current mode is replaced with whitespaces, thus
+preserving locations arriving from LSP intact."
+  (pm-with-synchronized-points
+    (save-excursion
+      (save-restriction
+        (widen)
+        (setq beg (or beg (point-min)))
+        (setq end (or end (point-max)))
+        (let ((cmode major-mode)
+              (end-eol (save-excursion (goto-char end)
+                                       (line-end-position)))
+              line-acc acc)
+          (pm-map-over-modes
+           (lambda (sbeg send)
+             (let ((beg1 (max sbeg beg))
+                   (end1 (min send end)))
+               (if (eq cmode major-mode)
+                   (progn
+                     (when (eq sbeg beg1)
+                       ;; first line of mode; use line-acc
+                       (setq acc (append line-acc acc))
+                       (setq line-acc nil))
+                     ;; if cur-mode follows after end on same line,
+                     ;; accumulate the last line but not the actual text
+                     (when (< beg1 end)
+                       (push (buffer-substring-no-properties beg1 end1) acc)))
+                 (goto-char beg1)
+                 (if (<= end1 (line-end-position))
+                     (when (< beg1 end1) ; don't accumulate on last line
+                       (push (make-string (- end1 beg1) ? ) line-acc))
+                   (while (< (line-end-position) end1)
+                     (push "\n" acc)
+                     (forward-line 1))
+                   (setq line-acc (list (make-string (- end1 (point)) ? )))))))
+           beg end-eol)
+          (apply #'concat (reverse acc)))))))
+
+;; We cannot compute original change location when modifications are complex
+;; (aka multiple changes are combined). In those cases we send an entire
+;; document.
+(defun pm--lsp-simple-change-p (beg len)
+  "Non-nil if the after change BEG and LEN match before change range."
+  (let ((bcr (pm--prop-get :before-change-range)))
+    (and (eq beg (car bcr))
+         (eq len (- (cdr bcr) (car bcr))))))
+
+(defun polymode-lsp-buffer-content (orig-fun)
+  "In polymode buffers, replace other modes' content with whitespaces.
+Use as around advice for lsp--buffer-content."
+  (if (and polymode-mode pm/polymode)
+      (pm--lsp-buffer-content)
+    (funcall orig-fun)))
+
+(defun polymode-lsp-change-event (orig-fun beg end len)
+  (if (and polymode-mode pm/polymode)
+      (pm--lsp-buffer-content-document-content-change-event beg end len)
+    (funcall orig-fun beg end len)))
+
+(defvar-local polymode-lsp-integration t
+  "Non-nil if lsp polymode integration should be enabled for this buffer.")
+
+(with-eval-after-load "lsp-mode"
+  (when polymode-lsp-integration
+    (add-to-list 'polymode-run-these-after-change-functions-in-other-buffers 'lsp-on-change)
+    ;; (add-to-list 'polymode-run-these-before-change-functions-in-other-buffers 'lsp-before-change)
+    ;; FIXME: add auto-save?
+    (add-to-list 'polymode-run-these-before-save-functions-in-other-buffers 'lsp--before-save)
+    (dolist (sym '(lsp-lens--after-save lsp-on-save))
+      (add-to-list 'polymode-run-these-after-save-functions-in-other-buffers sym))
+    ;; (add-to-list 'polymode-move-these-minor-modes-from-old-buffer 'lsp-headerline-breadcrumb-mode)
+    (pm-around-advice 'lsp--buffer-content #'polymode-lsp-buffer-content)
+    (pm-around-advice 'lsp--text-document-content-change-event #'polymode-lsp-change-event)))
+
 
 ;;; Flyspel
 (defun pm--flyspel-dont-highlight-in-chunkmodes (beg end _poss)
   (or (car (get-text-property beg :pm-span))
       (car (get-text-property end :pm-span))))
-
+(add-hook 'flyspell-incorrect-hook
+          #'pm--flyspel-dont-highlight-in-chunkmodes nil t)
 
 ;;; C/C++/Java
 (pm-around-advice 'c-before-context-fl-expand-region #'pm-override-output-cons)
@@ -280,33 +426,29 @@ changes."
 
 ;;; DESKTOP SAVE #194 #240
 
-;; NB: desktop-save will not save indirect buffer.
-;; For base buffer, if it's hidden as per #34, we will save it unhide by removing left whitespaces.
+;; NB: We advice desktop-save functionality to not save indirect buffers and for base buffers,
+;; save the buffers with un-hidden name.
 
 (defun polymode-fix-desktop-buffer-info (fn buffer)
-  "Unhide poly-mode base buffer which is hidden as per #34.
-This is done by modifying `uniquify-buffer-base-name' to `pm--core-buffer-name'."
+  "Unhide poly-mode base buffer which is hidden by removing
+the leading spaces from the name."
   (with-current-buffer buffer
-    (let ((out (funcall fn buffer))
-          (name (buffer-name)))
+    (let ((out (funcall fn buffer)))
       (when (and polymode-mode
                  (not (buffer-base-buffer))
                  (not (car out)))
-        (setf (car out) pm--core-buffer-name))
+        (setf (car out) (replace-regexp-in-string "^ +" "" (buffer-name buffer))))
       out)))
 
-(declare-function desktop-buffer-info "desktop")
-(with-eval-after-load "desktop"
-  (advice-add #'desktop-buffer-info :around #'polymode-fix-desktop-buffer-info))
-
 (defun polymode-fix-desktop-save-buffer-p (_ bufname &rest _args)
   "Dont save polymode buffers which are indirect buffers."
   (with-current-buffer bufname
-    (not (and polymode-mode
-              (buffer-base-buffer)))))
+    (not (and polymode-mode (buffer-base-buffer)))))
 
+(declare-function desktop-buffer-info "desktop")
 (declare-function desktop-save-buffer-p "desktop")
 (with-eval-after-load "desktop"
+  (advice-add #'desktop-buffer-info :around #'polymode-fix-desktop-buffer-info)
   (advice-add #'desktop-save-buffer-p :before-while #'polymode-fix-desktop-save-buffer-p))
 
 
@@ -321,14 +463,15 @@ This is done by modifying `uniquify-buffer-base-name' to `pm--core-buffer-name'.
 ;;; Undo Tree (#230)
 ;; Not clear why this fix works, or even why the problem occurs.
 (declare-function make-undo-tree "undo-tree")
+(defvar buffer-undo-tree)
 (defun polymode-init-undo-tree-maybe ()
   (when (and (boundp 'undo-tree-mode)
              undo-tree-mode
              (null buffer-undo-tree))
     (setq buffer-undo-tree (make-undo-tree))))
 
-(eval-after-load 'undo-tree
-  '(add-hook 'polymode-init-inner-hook 'polymode-init-undo-tree-maybe))
+(with-eval-after-load 'undo-tree
+  (add-hook 'polymode-init-inner-hook #'polymode-init-undo-tree-maybe))
 
 
 ;;; EVIL
@@ -342,8 +485,8 @@ This is done by modifying `uniquify-buffer-base-name' to `pm--core-buffer-name'.
         (with-current-buffer new-buffer
           (evil-change-state old-state))))))
 
-(eval-after-load 'evil-core
-  '(add-hook 'polymode-after-switch-buffer-hook 'polymode-switch-buffer-keep-evil-state-maybe))
+(with-eval-after-load 'evil-core
+  (add-hook 'polymode-after-switch-buffer-hook #'polymode-switch-buffer-keep-evil-state-maybe))
 
 
 ;;; HL line
@@ -359,8 +502,8 @@ This is done by modifying `uniquify-buffer-base-name' to `pm--core-buffer-name'.
       (hl-line-unhighlight))
     (when global-hl-line-mode
       (global-hl-line-unhighlight))))
-(eval-after-load 'hl-line
-  '(add-hook 'polymode-after-switch-buffer-hook 'polymode-switch-buffer-hl-unhighlight))
+(with-eval-after-load 'hl-line
+  (add-hook 'polymode-after-switch-buffer-hook #'polymode-switch-buffer-hl-unhighlight))
 
 
 ;;; YAS
@@ -374,14 +517,16 @@ This is done by modifying `uniquify-buffer-base-name' to `pm--core-buffer-name'.
 
 ;;; Multiple cursors
 
+(defvar mc--executing-command-for-fake-cursor)
 (defun polymode-disable-post-command-with-multiple-cursors (orig-fun &rest args)
   (unless mc--executing-command-for-fake-cursor
     (polymode-disable-post-command)
     (apply orig-fun args)
     (polymode-enable-post-command)))
 
-(with-eval-after-load "multiple-cursors-core"
-  (advice-add #'mc/execute-this-command-for-all-cursors :around
-              #'polymode-disable-post-command-with-multiple-cursors))
+(with-no-warnings
+  (with-eval-after-load "multiple-cursors-core"
+    (advice-add #'mc/execute-this-command-for-all-cursors :around
+                #'polymode-disable-post-command-with-multiple-cursors)))
 
 ;;; polymode-compat.el ends here
